#!/bin/bash
#
# Convertion d'une arborescence BDParcellaire standard en arborescence Geoportail
#
# v0.0.1 : didier.richard@ign.fr, le 2008-01-05 : first release
# v0.0.2 : didier.richard@ign.fr, le 2008-01-12 : enhancements

#set -x

# Constants :
VERSION="0.0.2"
CWDIR=`pwd`
SEP_BEGIN="\n--------------- Report :\n"
SEP_END="\n---------------\n"
DIR_TEST=gxt_dir
LOGS=$$.log
GCT=TILE.gct
SHP=000_GRD
REF1=expected_tile.txt
REF2=expected_tile.gxt
REF3=expected_tile.mif
REF4=expected_tile.mid
REF5=expected_$SHP.txt
REF6=expected_$SHP.gxt
REF7=expected_tile2.gxt
FOO=foo.gxt


# Global variables :
unset verbose
unset simul
unset path2apps
unset ogrinfo
unset ogr2ogr

#
# Usage :
usage () {
  cat >&2 <<EOF
usage: `basename $0` [ [--help, -h] | [--version, -v] ] |
                     [ [--verbose, -V] | [--simul, -s] ]
                     [PATH_2_OGR-GDAL_APPS_DIRECTORY]
EOF
exit $1
}

#
# Exec or simulate a command :
# $1 : commande
doCmd () {
  cmd2issue=$1
  [ -n "$simul" ] && echo "cmd:[$cmd2issue]" && return
  eval ${cmd2issue}
}

#
# Echo log if verbose
sendLOGS () {
  s="0"
  [ $# -eq 1 ]  && s=$1
  [ -n "$verbose" -a -f $LOGS ] && echo -e $SEP_BEGIN && cat $LOGS && echo -e $SEP_END
  doCmd "rm -f $LOGS"
  [ -n "$s" -a $s -ne 0 ] && exit $s
}

#
# main
#
while [ $# -gt 0 ] ; do
  case $1 in
  --help|-h)
    usage 0
    ;;
  --version|-v)
    cat >&2 <<EOF
`basename $0`, version ${VERSION}
EOF
    usage 0
    ;;
  --verbose|-V)
    verbose=true
    ;;
  --simul|-s)
    simul=true
    ;;
  *)
    if [ "z$path2apps" = "z" ] ; then
      path2apps="$1"
    fi
    ;;
  esac
  shift
done

if [ ! -n "$path2apps" ] ; then
  ogrinfo="ogrinfo"
  ogr2ogr="ogr2ogr"
else
  ogrinfo="$path2apps/ogrinfo"
  ogr2ogr="$path2apps/ogr2ogr"
fi

[ -n "$verbose" ] && export CPL_DEBUG="ON"

#----------
echo "Geoconcept unit tests : "
doCmd "rm -fr $DIR_TEST"

echo -n "  * ogrinfo on empty dir : "
doCmd "mkdir -p $DIR_TEST"
doCmd "$ogrinfo $DIR_TEST &>$LOGS"
[ ! -n "$simul" ] && {
  [ $(grep FAILURE $LOGS | wc -l) -eq 0 ] && {
    echo "failed - something wrong : "
    sendLOGS 100
  }
}
echo "succeed"
sendLOGS

#----------
echo -n "  * ogrinfo on non empty dir : "
doCmd "echo "Blah" > $DIR_TEST/$FOO"
doCmd "$ogrinfo $DIR_TEST &>$LOGS"
[ ! -n "$simul" ] && {
  [ $(grep FAILURE $LOGS | wc -l) -eq 0 ] && {
    echo "failed - something wrong : "
    sendLOGS 100
  }
}
echo "succeed"
sendLOGS
doCmd "rm -fr $DIR_TEST"

#----------
echo -n "  * ogrinfo on a non Geoconcept file : "
doCmd "echo "Blah" > $FOO"
doCmd "$ogrinfo $FOO &>$LOGS"
[ ! -n "$simul" ] && {
  [ $(grep FAILURE $LOGS | wc -l) -eq 0 ] && {
    echo "failed - something wrong : "
    sendLOGS 100
  }
}
echo "succeed"
sendLOGS
doCmd "rm -f $FOO"

#----------
echo -n "  * ogrinfo on a Geoconcept file (1) : "
doCmd "$ogrinfo -ro $REF1 &>$LOGS"
[ ! -n "$simul" ] && {
  [ $(grep FAILURE $LOGS | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    sendLOGS 100
  }
}
echo "succeed"
sendLOGS

#----------
echo -n "  * ogrinfo on a Geoconcept file (2) : "
doCmd "$ogrinfo -ro $REF1 TILE.TILE &>$LOGS"
[ ! -n "$simul" ] && {
  [ $(grep FAILURE $LOGS | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    sendLOGS 100
  }
}
echo "succeed"
sendLOGS

#----------
echo -n "  * creation test with config file : "
doCmd "rm -f tile.[gt]xt tile.mi[fd]"
doCmd "$ogr2ogr -f \"Geoconcept\" -a_srs \"+init=IGNF:LAMB93\" -dsco EXTENSION=txt -dsco CONFIG=$GCT tile.gxt $SHP.shp -lco FEATURETYPE=TILE.TILE"
[ ! -n "$simul" ] && {
  [ $(diff tile.txt $REF1 | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    diff tile.txt $REF1
    exit 1
  }
}
echo "succeed"
doCmd "mv tile.txt tile.gxt"

#----------
echo -n "  * append test : "
doCmd "$ogr2ogr -f \"Geoconcept\" -update -append tile.gxt $SHP.shp -nln TILE.TILE"
[ ! -n "$simul" ] && {
  [ $(diff tile.gxt $REF2 | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    diff tile.gxt $REF2
    exit 1
  }
}
echo "succeed"

#----------
echo -n "  * read test : "
doCmd "$ogr2ogr -f \"MapInfo File\" -dsco FORMAT=MIF tile.mif tile.gxt TILE.TILE"
[ ! -n "$simul" ] && {
  [ $(diff tile.mif $REF3 | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    diff tile.mif $REF3
    exit 1
  }
  [ $(diff tile.mid $REF4 | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    diff tile.mif $REF4
    exit 1
  }
}
echo "succeed"
doCmd "rm -f tile.[gt]xt tile.mi[fd] $SHP.[gt]xt"

#----------
echo -n "  * creation test without config file : "
doCmd "$ogr2ogr -f \"Geoconcept\" -a_srs \"+init=IGNF:LAMB93\" -dsco EXTENSION=txt $SHP.txt $SHP.shp"
[ ! -n "$simul" ] && {
  [ $(diff $SHP.txt $REF5 | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    diff $SHP.txt $REF5
    exit 1
  }
}
echo "succeed"
doCmd "mv $SHP.txt $SHP.gxt"

#----------
echo -n "  * append test : "
doCmd "$ogr2ogr -f \"Geoconcept\" -update -append $SHP.gxt $SHP.shp"
[ ! -n "$simul" ] && {
  [ $(diff $SHP.gxt $REF6 | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    diff $SHP.gxt $REF6
    exit 1
  }
}
echo "succeed"

#----------
echo -n "  * ogrinfo on a Geoconcept file (3) : "
doCmd "$ogrinfo -ro $SHP.gxt $SHP.$SHP &>$LOGS"
[ ! -n "$simul" ] && {
  [ $(grep FAILURE $LOGS | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    sendLOGS 100
  }
}
echo "succeed"
sendLOGS
doCmd "rm -f $SHP.gxt"

#----------
echo -n "  * creation test with config file (TILE.TILE) : "
doCmd "$ogr2ogr -f \"Geoconcept\" -a_srs \"+init=IGNF:LAMB93\" -dsco CONFIG=$GCT tile.gxt $SHP.shp -lco FEATURETYPE=TILE.TILE"
echo "succeed"

#----------
echo -n "  * append test ($SHP.$SHP) : "
doCmd "$ogr2ogr -f \"Geoconcept\" -update -append tile.gxt $SHP.shp"
[ ! -n "$simul" ] && {
  [ $(diff tile.gxt $REF7 | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    diff tile.gxt $REF7
    exit 1
  }
}
echo "succeed"

#----------
echo -n "  * ogrinfo on a Geoconcept file (4) : "
doCmd "$ogrinfo -ro tile.gxt TILE.TILE $SHP.$SHP 2>$LOGS"
[ ! -n "$simul" ] && {
  [ $(grep FAILURE $LOGS | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    sendLOGS 100
  }
}
echo "succeed"
sendLOGS

#----------
echo -n "  * ogrinfo -spat 595000 7130000 1145000 6580000 : "
doCmd "$ogrinfo -spat 595000 7130000 1145000 6580000 tile.gxt TILE.TILE 2>$LOGS"
[ ! -n "$simul" ] && {
  [ $(grep FAILURE $LOGS | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    sendLOGS 100
  }
}
echo "succeed"
sendLOGS

#----------
echo -n "  * ogrinfo -where idSel='000-2007-0595-6585-LAMB93' : "
doCmd "$ogrinfo -where idSel='000-2007-0595-6585-LAMB93' tile.gxt TILE.TILE $SHP.$SHP 2>$LOGS"
[ ! -n "$simul" ] && {
  [ $(grep FAILURE $LOGS | wc -l) -ne 0 ] && {
    echo "failed - something wrong : "
    sendLOGS 100
  }
}
echo "succeed"
sendLOGS
doCmd "rm -f tile.gxt"

#----------
[ -n "$verbose" ] && unset CPL_DEBUG
echo "Geoconcept unit tests terminated."

exit 0
